require "rover_utils"
require "rover_environment"
require "acm"
require "relay_board"
require "singleton"

#This class is a singleton, to access it call 'Rover.instance'
class Rover
  include Singleton
  
  attr_accessor :acm, :relay

  #in the future, this should be where a config file is read to initialize
  #all the rover's subsystems.
  def initialize
    self.relay = RelayBoard.new nil, "/dev/relay"
    self.acm = Acm.new nil, "/dev/acm"
    #self.acm = Tcm2Compass.new nil
  end
  
end
