require 'acm'

def acm_test
  cm = Acm.new nil

  cm.run

  while true
    sleep(1)
    head = cm.heading
    avg  = cm.average_heading
    time = cm.time
    puts "Time=#{time}\tHeading=#{head}\tAvg Heading=#{avg}"
  end
end

acm_test
