require 'ezservo'
require 'tcm2_compass'
require 'control_system'
require 'relay_board'

module Myirb
  def init
    @rb = RelayBoard.new nil
    @rb.turn_on 1
    sleep 1
    @ez = Ezservo.mini_rover
    @cs = ControlSystem.new @ez
    @tcm = Tcm2Compass.new nil
  end
end
