class Compass
  def initialize
    @headings = [190, 300, 90, 350, 5, 0, 0, 0, 0, 0, 0, 0]
    @heading_count = 0
  end
  
  def get_heading
    heading = @headings[@heading_count]
    @heading_count += 1
    heading
  end
end
