require 'ezservo'
require 'relay_board'

rb = RelayBoard.new nil, "/dev/relay"
rb.turn_on 0
sleep 1

ez = Ezservo.new nil, "/dev/ezservo"
ez.serial_port.write "/As0w1000x0y2500V2000R\r"

sleep 1
rb.turn_off 0
